package com.ruoyi.buyerapi.service;

import java.util.List;
import java.util.Map;

import com.ruoyi.buyerapi.domain.BuyerEmpWorkhour;
import com.ruoyi.buyerapi.domain.BuyerEmpWorkhourByProject;

/**
 * 员工工时分摊记录Service接口
 *
 * @author ruoyi
 * @date 2022-09-22
 */
public interface IBuyerEmpWorkhourService
{
    /**
     * 查询员工工时分摊记录列表
     *
     * @param BuyerEmpWorkhourByProject 员工工时分摊记录
     * @return 员工工时分摊记录集合
     */
    public List<BuyerEmpWorkhourByProject> listGrroupProject(BuyerEmpWorkhourByProject buyerEmpWorkhourByProject);

    /**
     * 查询员工工时分摊记录列表
     *
     * @param projectId 项目id
     * @return 员工工时分摊记录集合
     */
    public List<BuyerEmpWorkhourByProject> listByProjectId(String projectId);

    /**
     * 查询员工工时分摊记录
     *
     * @param id 员工工时分摊记录主键
     * @return 员工工时分摊记录
     */
    public BuyerEmpWorkhour selectBuyerEmpWorkhourById(String id);

    /**
     * 查询员工工时分摊记录列表
     *
     * @param buyerEmpWorkhour 员工工时分摊记录
     * @return 员工工时分摊记录集合
     */
    public List<BuyerEmpWorkhour> selectBuyerEmpWorkhourList(BuyerEmpWorkhour buyerEmpWorkhour);

    /**
     * 查询员工工时分摊记录列表
     *
     * @param buyerEmpWorkhour 员工工时分摊记录
     * @return 员工工时分摊记录集合
     */
    public List<Map<String, Object>> listForYearMonth(BuyerEmpWorkhour buyerEmpWorkhour);

    /**
     * 查询员工工时分摊记录列表
     *
     * @param buyerEmpWorkhour 员工工时分摊记录
     * @return 员工工时分摊记录集合
     */
    public List<String> workDatelist(BuyerEmpWorkhour buyerEmpWorkhour);

    /**
     * 新增员工工时分摊记录
     *
     * @param buyerEmpWorkhour 员工工时分摊记录
     * @return 结果
     */
    public int insertBuyerEmpWorkhour(BuyerEmpWorkhour buyerEmpWorkhour);

    /**
     * 修改员工工时分摊记录
     *
     * @param buyerEmpWorkhour 员工工时分摊记录
     * @return 结果
     */
    public int updateBuyerEmpWorkhour(BuyerEmpWorkhour buyerEmpWorkhour);

    /**
     * 批量删除员工工时分摊记录
     *
     * @param ids 需要删除的员工工时分摊记录主键集合
     * @return 结果
     */
    public int deleteBuyerEmpWorkhourByIds(String[] ids);

    /**
     * 删除员工工时分摊记录信息
     *
     * @param id 员工工时分摊记录主键
     * @return 结果
     */
    public int deleteBuyerEmpWorkhourById(String id);

    /**
     * 删除员工工时分摊记录信息
     *
     * @param buyerEmpWorkhour 员工工时对象
     * @return 结果
     */
    public int deleteBuyerEmpWorkhour(BuyerEmpWorkhour buyerEmpWorkhour);
}
